@@ -45,14 +45,20 @@ pub struct Cli {
     pub httpie_compat_mode: bool,
 
     /// (default) Serialize data items from the command line as a JSON object.
+    ///
+    /// Overrides both --form and --multipart.
     #[clap(short = 'j', long, overrides_with_all = &["form", "multipart"])]
     pub json: bool,
 
     /// Serialize data items from the command line as form fields.
+    ///
+    /// Overrides both --json and --multipart.
     #[clap(short = 'f', long, overrides_with_all = &["json", "multipart"])]
     pub form: bool,
 
     /// Like --form, but force a multipart/form-data request even without files.
+    ///
+    /// Overrides both --json and --form.
     #[clap(short = 'm', long, overrides_with_all = &["json", "form"])]
     pub multipart: bool,
 
@@ -60,8 +66,21 @@ pub struct Cli {
     #[clap(long, value_name = "RAW")]
     pub raw: Option<String>,
 
-    /// Controls output processing.
-    #[clap(long, arg_enum, value_name = "STYLE")]
+    /// Controls output processing
+    #[clap(
+        long,
+        arg_enum,
+        value_name = "STYLE",
+        long_help = "\
+Controls output processing. Possible values are:
+
+    all      (default) Enable both coloring and formatting
+    colors   Apply syntax highlighting to output
+    format   Pretty-print json and sort headers
+    none     Disable both coloring and formatting
+
+Defaults to \"format\" if the NO_COLOR env is set and to \"none\" if stdout is not tty."
+    )]
     pub pretty: Option<Pretty>,
 
     /// Output coloring style.
@@ -70,22 +89,22 @@ pub struct Cli {
 
     /// Override the response encoding for terminal display purposes.
     ///
-    /// Example: `--response-charset=latin1`
+    /// Example: --response-charset=latin1
     #[clap(long, value_name = "ENCODING", parse(try_from_str = parse_encoding))]
     pub response_charset: Option<&'static Encoding>,
 
-    /// Override the response mime type for coloring and formatting for the terminal
+    /// Override the response mime type for coloring and formatting for the terminal.
     ///
-    /// Example: `--response-mime=application/json`
+    /// Example: --response-mime=application/json
     #[clap(long, value_name = "MIME_TYPE")]
     pub response_mime: Option<String>,
 
     /// String specifying what the output should contain.
     ///
-    /// Use `H` and `B` for request header and body respectively,
-    /// and `h` and `b` for response hader and body.
+    /// Use "H" and "B" for request header and body respectively,
+    /// and "h" and "b" for response header and body.
     ///
-    /// Example: `--print=Hb`
+    /// Example: --print=Hb
     #[clap(short = 'p', long, value_name = "FORMAT")]
     pub print: Option<Print>,
 
@@ -98,6 +117,11 @@ pub struct Cli {
     pub body: bool,
 
     /// Print the whole request as well as the response.
+    ///
+    /// Additionally, this enables --all for printing intermediary
+    /// requests/responses while following redirects.
+    ///
+    /// Equivalent to --print=HhBb --all.
     #[clap(short = 'v', long)]
     pub verbose: bool,
 
@@ -109,6 +133,14 @@ pub struct Cli {
     #[clap(short = 'P', long, value_name = "FORMAT")]
     pub history_print: Option<Print>,
 
+    /// Resolve hostname to ipv4 addresses only.
+    #[clap(short = '4', long)]
+    pub ipv4: bool,
+
+    /// Resolve hostname to ipv6 addresses only.
+    #[clap(short = '6', long)]
+    pub ipv6: bool,
+
     /// Do not print to stdout or stderr.
     #[clap(short = 'q', long)]
     pub quiet: bool,
@@ -122,6 +154,8 @@ pub struct Cli {
     pub output: Option<PathBuf>,
 
     /// Download the body to a file instead of printing it.
+    ///
+    /// The Accept-Encoding header is set to identify and any redirects will be followed.
     #[clap(short = 'd', long)]
     pub download: bool,
 
@@ -158,12 +192,12 @@ pub struct Cli {
     #[clap(short = 'A', long, arg_enum)]
     pub auth_type: Option<AuthType>,
 
-    /// Authenticate as USER with PASS or with TOKEN.
+    /// Authenticate as USER with PASS (-A basic|digest) or with TOKEN (-A bearer).
     ///
-    /// PASS will be prompted if missing. Use a trailing colon (i.e. `USER:`)
+    /// PASS will be prompted if missing. Use a trailing colon (i.e. "USER:")
     /// to authenticate with just a username.
     ///
-    /// TOKEN is expected if `--auth-type=bearer`.
+    /// TOKEN is expected if --auth-type=bearer.
     #[clap(short = 'a', long, value_name = "USER[:PASS] | TOKEN")]
     pub auth: Option<String>,
 
@@ -195,26 +229,26 @@ pub struct Cli {
     #[clap(short = 'F', long)]
     pub follow: bool,
 
-    /// Number of redirects to follow, only respected if `follow` is set.
+    /// Number of redirects to follow. Only respected if --follow is used.
     #[clap(long, value_name = "NUM")]
     pub max_redirects: Option<usize>,
 
     /// Connection timeout of the request.
     ///
-    /// The default value is `0`, i.e., there is no timeout limit.
+    /// The default value is "0", i.e., there is no timeout limit.
     #[clap(long, value_name = "SEC")]
     pub timeout: Option<Timeout>,
 
-    /// Use a proxy for a protocol. For example: `--proxy https:http://proxy.host:8080`.
+    /// Use a proxy for a protocol. For example: --proxy https:http://proxy.host:8080.
     ///
-    /// PROTOCOL can be `http`, `https` or `all`.
+    /// PROTOCOL can be "http", "https" or "all".
     ///
     /// If your proxy requires credentials, put them in the URL, like so:
-    /// `--proxy http:socks5://user:password@proxy.host:8000`.
+    /// --proxy http:socks5://user:password@proxy.host:8000.
     ///
     /// You can specify proxies for multiple protocols by repeating this option.
     ///
-    /// The environment variables `http_proxy` and `https_proxy` can also be used, but
+    /// The environment variables "http_proxy" and "https_proxy" can also be used, but
     /// are completely ignored if --proxy is passed.
     #[clap(long, value_name = "PROTOCOL:URL", number_of_values = 1)]
     pub proxy: Vec<Proxy>,
@@ -250,7 +284,7 @@ pub struct Cli {
     pub ssl: Option<std::option::Option<tls::Version>>,
 
     /// Use the system TLS library instead of rustls (if enabled at compile time).
-    #[clap(long)]
+    #[clap(long, hide = cfg!(not(all(feature = "native-tls", feature = "rustls"))))]
     pub native_tls: bool,
 
     /// The default scheme to use if not specified in the URL.
@@ -261,7 +295,7 @@ pub struct Cli {
     #[clap(long)]
     pub https: bool,
 
-    /// HTTP version to use
+    /// HTTP version to use.
     #[clap(long, value_name = "VERSION",
         possible_value = clap::PossibleValue::new("1.0"),
         possible_value = clap::PossibleValue::new("1.1").alias("1"),
@@ -270,10 +304,16 @@ pub struct Cli {
     pub http_version: Option<HttpVersion>,
 
     /// Do not attempt to read stdin.
+    ///
+    /// This disables the default behaviour of reading the request body from stdin
+    /// when a redirected input is detected.
+    ///
+    /// It is recommended to pass this flag when using xh for scripting purposes.
+    /// For more information, refer to https://httpie.io/docs/cli/best-practices.
     #[clap(short = 'I', long)]
     pub ignore_stdin: bool,
 
-    /// Print a translation to a `curl` command.
+    /// Print a translation to a curl command.
     ///
     /// For translating the other way, try https://curl2httpie.online/.
     #[clap(long)]
@@ -285,24 +325,65 @@ pub struct Cli {
 
     /// The request URL, preceded by an optional HTTP method.
     ///
-    /// METHOD can be `get`, `post`, `head`, `put`, `patch`, `delete` or `options`.
-    /// If omitted, either a GET or a POST will be done depending on whether the
-    /// request sends data.
+    /// If the method is omitted, it will default to GET, or to POST
+    /// if the request contains a body.
+    ///
+    /// The URL scheme defaults to "http://" normally, or "https://" if
+    /// the program is invoked as "xhs".
+    ///
+    /// A leading colon works as shorthand for localhost. ":8000" is equivalent
+    /// to "localhost:8000", and ":/path" is equivalent to "localhost/path".
     #[clap(value_name = "[METHOD] URL")]
     raw_method_or_url: String,
 
-    /// Optional key-value pairs to be included in the request
+    /// Optional key-value pairs to be included in the request.
     ///
-    ///   - key==value to add a parameter to the URL
-    ///   - key=value to add a JSON field (--json) or form field (--form)
-    ///   - key:=value to add a complex JSON value (e.g. `numbers:=[1,2,3]`)
-    ///   - key@filename to upload a file from filename (with --form)
-    ///   - @filename to use a file as the request body
-    ///   - header:value to add a header
-    ///   - header: to unset a header
-    ///   - header; to add a header with an empty value
+    /// The separator is used to determine the type:
     ///
-    /// A backslash can be used to escape special characters (e.g. weird\:key=value).
+    ///     key==value
+    ///         Add a query string to the URL.
+    ///
+    ///     key=value
+    ///         Add a JSON property (--json) or form field (--form) to
+    ///         the request body.
+    ///
+    ///     key=@filename
+    ///         Add a JSON property (--json) or form field (--form) from a
+    ///         file to the request body.
+    ///
+    ///     key:=value
+    ///         Add a field with a literal JSON value to the request body.
+    ///
+    ///         Example: "numbers:=[1,2,3] enabled:=true"
+    ///
+    ///     key:=@filename
+    ///         Add a field with a literal JSON value from a file to the
+    ///         request body.
+    ///
+    ///     key@filename
+    ///         Upload a file (requires --form or --multipart).
+    ///
+    ///         To set the filename and mimetype, ";type=" and
+    ///         ";filename=" can be used respectively.
+    ///         
+    ///         Example: "pfp@ra.jpg;type=image/jpeg;filename=profile.jpg"
+    ///
+    ///     @filename
+    ///         Use a file as the request body.
+    ///
+    ///     header:value
+    ///         Add a header, e.g. "user-agent:foobar"
+    ///
+    ///     header:
+    ///         Unset a header, e.g. "connection:"
+    ///
+    ///     header;
+    ///         Add a header with an empty value.
+    ///
+    /// A backslash can be used to escape special characters, e.g. "weird\:key=value".
+    ///
+    /// To construct a complex JSON object, the REQUEST_ITEM's key can be set to a JSON path instead of a field name.
+    /// For more information on this syntax, refer to https://httpie.io/docs/cli/nested-json.
     #[clap(value_name = "REQUEST_ITEM", verbatim_doc_comment)]
     raw_rest_args: Vec<String>,
 
@@ -357,28 +438,11 @@ impl Cli {
                 // would print long help and print short help instead. And if we do
                 // want to print long help, then we handle that in try_parse_from
                 // instead of here.
-                if env::var_os("XH_HELP2MAN").is_some() {
-                    Self::into_app()
-                        .help_template(
-                            "\
-                                Usage: {usage}\n\
-                                \n\
-                                {about}\n\
-                                \n\
-                                Options:\n\
-                                {options}\n\
-                                {after-help}\
-                            ",
-                        )
-                        .print_long_help()
-                        .unwrap();
-                } else {
-                    Self::into_app().print_help().unwrap();
-                    println!(
-                        "\nRun `{} help` for more complete documentation.",
-                        env!("CARGO_PKG_NAME")
-                    );
-                }
+                Self::into_app().print_help().unwrap();
+                println!(
+                    "\nRun `{} help` for more complete documentation.",
+                    env!("CARGO_PKG_NAME")
+                );
                 safe_exit();
             }
             Err(err) => err.exit(),
@@ -396,12 +460,16 @@ impl Cli {
 
         match cli.raw_method_or_url.as_str() {
             "help" => {
+                // opt-out of clap's auto-generated possible values help for --pretty
+                // as we already list them in the long_help
+                app = app.mut_arg("pretty", |a| a.hide_possible_values(true));
+
                 app.print_long_help().unwrap();
                 println!();
                 safe_exit();
             }
-            "print_completions" => return Err(print_completions(app, cli.raw_rest_args)),
-            "generate_completions" => return Err(generate_completions(app, cli.raw_rest_args)),
+            "generate-completions" => return Err(generate_completions(app, cli.raw_rest_args)),
+            "generate-manpages" => return Err(generate_manpages(app, cli.raw_rest_args)),
             _ => {}
         }
         let mut rest_args = mem::take(&mut cli.raw_rest_args).into_iter();
@@ -454,6 +522,10 @@ impl Cli {
             )
         })?;
 
+        if cfg!(not(feature = "rustls")) {
+            cli.native_tls = true;
+        }
+
         Ok(cli)
     }
 
@@ -469,7 +541,7 @@ impl Cli {
             self.default_scheme = Some("https".to_string());
         }
         if self.bearer.is_some() {
-            self.auth_type = Some(AuthType::bearer);
+            self.auth_type = Some(AuthType::Bearer);
             self.auth = self.bearer.take();
         }
         self.check_status = match (self.check_status_raw, matches.is_present("no-check-status")) {
@@ -619,32 +691,8 @@ fn construct_url(
     Ok(url)
 }
 
-// This signature is a little weird: we either return an error or don't
-// return at all
-fn print_completions(mut app: clap::Command, rest_args: Vec<String>) -> Error {
-    let bin_name = match app.get_bin_name() {
-        // This name is borrowed from `app`, and `gen_completions_to()` mutably
-        // borrows `app`, so we need to do a clone
-        Some(name) => name.to_owned(),
-        None => return app.error(ErrorKind::EmptyValue, "Missing binary name"),
-    };
-    if rest_args.len() != 1 {
-        return app.error(
-            ErrorKind::WrongNumberOfValues,
-            "Usage: xh print_completions <SHELL>",
-        );
-    }
-    let shell = match rest_args[0].parse::<clap_complete::Shell>() {
-        Ok(shell) => shell,
-        Err(_) => return app.error(ErrorKind::InvalidValue, "Unknown shell name"),
-    };
-    let mut buf = Vec::new();
-    clap_complete::generate(shell, &mut app, bin_name, &mut buf);
-    let completions = String::from_utf8(buf).unwrap();
-    print!("{}", completions);
-    safe_exit();
-}
-
+#[cfg(feature = "man-completion-gen")]
+// This signature is a little weird: we either return an error or don't return at all
 fn generate_completions(mut app: clap::Command, rest_args: Vec<String>) -> Error {
     let bin_name = match app.get_bin_name() {
         Some(name) => name.to_owned(),
@@ -653,12 +701,12 @@ fn generate_completions(mut app: clap::Command, rest_args: Vec<String>) -> Error
     if rest_args.len() != 1 {
         return app.error(
             ErrorKind::WrongNumberOfValues,
-            "Usage: xh generate_completions <DIRECTORY>",
+            "Usage: xh generate-completions <DIRECTORY>",
         );
     }
+
     for &shell in clap_complete::Shell::value_variants() {
         // Elvish complains about multiple deprecations and these don't seem to work
-        // If you must use them, generate them manually with xh print_completions elvish
         if shell != clap_complete::Shell::Elvish {
             clap_complete::generate_to(shell, &mut app, &bin_name, &rest_args[0]).unwrap();
         }
@@ -666,28 +714,214 @@ fn generate_completions(mut app: clap::Command, rest_args: Vec<String>) -> Error
     safe_exit();
 }
 
-#[allow(non_camel_case_types)]
-#[derive(ArgEnum, Copy, Clone, Debug, PartialEq)]
+#[cfg(feature = "man-completion-gen")]
+fn generate_manpages(mut app: clap::Command, rest_args: Vec<String>) -> Error {
+    use roff::{bold, italic, roman, Roff};
+    use time::OffsetDateTime as DateTime;
+
+    if rest_args.len() != 1 {
+        return app.error(
+            ErrorKind::WrongNumberOfValues,
+            "Usage: xh generate-manpages <DIRECTORY>",
+        );
+    }
+
+    let items: Vec<_> = app.get_arguments().filter(|i| !i.is_hide_set()).collect();
+
+    let mut request_items_roff = Roff::new();
+    let request_items = items
+        .iter()
+        .find(|opt| opt.get_id() == "raw-rest-args")
+        .unwrap();
+    let request_items_help = request_items
+        .get_long_help()
+        .or_else(|| request_items.get_help())
+        .expect("request_items is missing help");
+
+    // replace the indents in request_item help with proper roff controls
+    // For example:
+    //
+    // ```
+    // normal help normal help
+    // normal help normal help
+    //
+    //   request-item-1
+    //     help help
+    //
+    //   request-item-2
+    //     help help
+    //
+    // normal help normal help
+    // ```
+    //
+    // Should look like this with roff controls
+    //
+    // ```
+    // normal help normal help
+    // normal help normal help
+    // .RS 12
+    // .TP
+    // request-item-1
+    // help help
+    // .TP
+    // request-item-2
+    // help help
+    // .RE
+    //
+    // .RS
+    // normal help normal help
+    // .RE
+    // ```
+    let lines: Vec<&str> = request_items_help.lines().collect();
+    let mut rs = false;
+    for i in 0..lines.len() {
+        if lines[i].is_empty() {
+            let prev = lines[i - 1].chars().take_while(|&x| x == ' ').count();
+            let next = lines[i + 1].chars().take_while(|&x| x == ' ').count();
+            if prev != next && next > 0 {
+                if !rs {
+                    request_items_roff.control("RS", ["8"]);
+                    rs = true;
+                }
+                request_items_roff.control("TP", ["4"]);
+            } else if prev != next && next == 0 {
+                request_items_roff.control("RE", []);
+                request_items_roff.text(vec![roman("")]);
+                request_items_roff.control("RS", []);
+            } else {
+                request_items_roff.text(vec![roman(lines[i])]);
+            }
+        } else {
+            request_items_roff.text(vec![roman(lines[i].trim())]);
+        }
+    }
+    request_items_roff.control("RE", []);
+
+    let mut options_roff = Roff::new();
+    let non_pos_items = items
+        .iter()
+        .filter(|a| !a.is_positional())
+        .collect::<Vec<_>>();
+    // move the first two items (i.e. --help, --version) to the end
+    let non_pos_items = non_pos_items
+        .iter()
+        .cycle()
+        .skip(2)
+        .take(non_pos_items.len());
+
+    for opt in non_pos_items {
+        let mut header = vec![];
+        if let Some(short) = opt.get_short() {
+            header.push(bold(format!("-{}", short)));
+        }
+        if let Some(long) = opt.get_long() {
+            if !header.is_empty() {
+                header.push(roman(", "));
+            }
+            header.push(bold(format!("--{}", long)));
+        }
+        if let Some(value) = &opt.get_value_names() {
+            if opt.get_long().is_some() {
+                header.push(roman("="));
+            } else {
+                header.push(roman(" "));
+            }
+
+            if opt.get_id() == "auth" {
+                header.push(italic("USER"));
+                header.push(roman("["));
+                header.push(italic(":PASS"));
+                header.push(roman("] | "));
+                header.push(italic("TOKEN"));
+            } else {
+                header.push(italic(&value.join(" ")));
+            }
+        }
+        let mut body = vec![];
+
+        let mut help = opt
+            .get_long_help()
+            .or_else(|| opt.get_help())
+            .expect("option is missing help")
+            .to_owned();
+        if !help.ends_with('.') {
+            help.push('.')
+        }
+        body.push(roman(help));
+
+        if let Some(possible_values) = opt.get_possible_values() {
+            if !opt.is_hide_possible_values_set() && opt.get_id() != "pretty" {
+                let possible_values_text = format!(
+                    "\n\n[possible values: {}]",
+                    possible_values
+                        .iter()
+                        .map(|v| v.get_name())
+                        .collect::<Vec<_>>()
+                        .join(", ")
+                );
+                body.push(roman(possible_values_text));
+            }
+        }
+        options_roff.control("TP", ["4"]);
+        options_roff.text(header);
+        options_roff.text(body);
+    }
+
+    let mut manpage = fs::read_to_string(format!("{}/man-template.roff", rest_args[0])).unwrap();
+
+    let current_date = {
+        let (year, month, day) = DateTime::now_utc().date().as_ymd();
+        format!("{}-{:02}-{:02}", year, month, day)
+    };
+
+    manpage = manpage.replace("{{date}}", &current_date);
+    manpage = manpage.replace("{{version}}", app.get_version().unwrap());
+    manpage = manpage.replace("{{request_items}}", request_items_roff.to_roff().trim());
+    manpage = manpage.replace("{{options}}", options_roff.to_roff().trim());
+
+    fs::write(format!("{}/xh.1", rest_args[0]), manpage).unwrap();
+    safe_exit();
+}
+
+#[cfg(not(feature = "man-completion-gen"))]
+fn generate_completions(mut _app: clap::Command, _rest_args: Vec<String>) -> Error {
+    clap::Error::raw(
+        clap::ErrorKind::InvalidSubcommand,
+        "generate-completions requires enabling man-completion-gen feature\n",
+    )
+}
+
+#[cfg(not(feature = "man-completion-gen"))]
+fn generate_manpages(mut _app: clap::Command, _rest_args: Vec<String>) -> Error {
+    clap::Error::raw(
+        clap::ErrorKind::InvalidSubcommand,
+        "generate-manpages requires enabling man-completion-gen feature\n",
+    )
+}
+
+#[derive(ArgEnum, Copy, Clone, Debug, PartialEq, Eq)]
 pub enum AuthType {
-    basic,
-    bearer,
-    digest,
+    Basic,
+    Bearer,
+    Digest,
 }
 
 impl Default for AuthType {
     fn default() -> Self {
-        AuthType::basic
+        AuthType::Basic
     }
 }
 
-// Uppercase variant names would show up as such in the help text
-#[allow(non_camel_case_types)]
-#[derive(ArgEnum, Debug, PartialEq, Clone, Copy)]
+#[derive(ArgEnum, Debug, PartialEq, Eq, Clone, Copy)]
 pub enum Pretty {
-    all,
-    colors,
-    format,
-    none,
+    /// (default) Enable both coloring and formatting
+    All,
+    /// Apply syntax highlighting to output
+    Colors,
+    /// Pretty-print json and sort headers
+    Format,
+    /// Disable both coloring and formatting
+    None,
 }
 
 /// The caller must check in advance if the string is valid. (clap does this.)
@@ -705,30 +939,29 @@ fn parse_tls_version(text: &str) -> Option<tls::Version> {
 
 impl Pretty {
     pub fn color(self) -> bool {
-        matches!(self, Pretty::colors | Pretty::all)
+        matches!(self, Pretty::Colors | Pretty::All)
     }
 
     pub fn format(self) -> bool {
-        matches!(self, Pretty::format | Pretty::all)
+        matches!(self, Pretty::Format | Pretty::All)
     }
 }
 
-#[allow(non_camel_case_types)]
-#[derive(ArgEnum, Debug, PartialEq, Clone, Copy)]
+#[derive(ArgEnum, Debug, PartialEq, Eq, Clone, Copy)]
 pub enum Theme {
-    auto,
-    solarized,
-    monokai,
-    fruity,
+    Auto,
+    Solarized,
+    Monokai,
+    Fruity,
 }
 
 impl Theme {
     pub fn as_str(&self) -> &'static str {
         match self {
-            Theme::auto => "ansi",
-            Theme::solarized => "solarized",
-            Theme::monokai => "monokai",
-            Theme::fruity => "fruity",
+            Theme::Auto => "ansi",
+            Theme::Solarized => "solarized",
+            Theme::Monokai => "monokai",
+            Theme::Fruity => "fruity",
         }
     }
 }
@@ -847,7 +1080,7 @@ impl FromStr for Timeout {
     }
 }
 
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Eq)]
 pub enum Proxy {
     Http(Url),
     Https(Url),
@@ -884,7 +1117,7 @@ impl FromStr for Proxy {
     }
 }
 
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Eq)]
 pub enum Verify {
     Yes,
     No,
@@ -914,7 +1147,7 @@ impl fmt::Display for Verify {
     }
 }
 
-#[derive(Debug, PartialEq, Copy, Clone)]
+#[derive(Debug, PartialEq, Eq, Copy, Clone)]
 pub enum BodyType {
     Json,
     Form,
@@ -972,7 +1205,7 @@ fn parse_encoding(encoding: &str) -> anyhow::Result<&'static Encoding> {
         _ => (),
     }
 
-    for encoding in &[
+    for encoding in [
         &normalized_encoding,
         &normalized_encoding.replace(&['-', '_'][..], ""),
         &normalized_encoding.replace('_', "-"),
@@ -985,7 +1218,7 @@ fn parse_encoding(encoding: &str) -> anyhow::Result<&'static Encoding> {
 
     {
         let mut encoding = normalized_encoding.replace(&['-', '_'][..], "");
-        if let Some(first_digit_index) = encoding.find(|c: char| c.is_digit(10)) {
+        if let Some(first_digit_index) = encoding.find(|c: char| c.is_ascii_digit()) {
             encoding.insert(first_digit_index, '-');
             if let Some(encoding) = Encoding::for_label(encoding.as_bytes()) {
                 return Ok(encoding);
@@ -1017,17 +1250,21 @@ mod tests {
 
     use crate::request_items::RequestItem;
 
-    fn parse(args: &[&str]) -> Result<Cli> {
+    fn parse<I>(args: I) -> Result<Cli>
+    where
+        I: IntoIterator,
+        I::Item: Into<OsString> + Clone,
+    {
         Cli::try_parse_from(
-            Some("xh".to_string())
+            Some("xh".into())
                 .into_iter()
-                .chain(args.iter().map(ToString::to_string)),
+                .chain(args.into_iter().map(Into::into)),
         )
     }
 
     #[test]
     fn implicit_method() {
-        let cli = parse(&["example.org"]).unwrap();
+        let cli = parse(["example.org"]).unwrap();
         assert_eq!(cli.method, None);
         assert_eq!(cli.url.to_string(), "http://example.org/");
         assert!(cli.request_items.items.is_empty());
@@ -1035,7 +1272,7 @@ mod tests {
 
     #[test]
     fn explicit_method() {
-        let cli = parse(&["get", "example.org"]).unwrap();
+        let cli = parse(["get", "example.org"]).unwrap();
         assert_eq!(cli.method, Some(Method::GET));
         assert_eq!(cli.url.to_string(), "http://example.org/");
         assert!(cli.request_items.items.is_empty());
@@ -1044,25 +1281,25 @@ mod tests {
     #[test]
     fn method_edge_cases() {
         // "localhost" is interpreted as method; this is undesirable, but expected
-        parse(&["localhost"]).unwrap_err();
+        parse(["localhost"]).unwrap_err();
 
         // Non-standard method used by varnish
-        let cli = parse(&["purge", ":"]).unwrap();
+        let cli = parse(["purge", ":"]).unwrap();
         assert_eq!(cli.method, Some("PURGE".parse().unwrap()));
         assert_eq!(cli.url.to_string(), "http://localhost/");
 
         // Zero-length arg should not be interpreted as method, but fail to parse as URL
-        parse(&[""]).unwrap_err();
+        parse([""]).unwrap_err();
     }
 
     #[test]
     fn missing_url() {
-        parse(&["get"]).unwrap_err();
+        parse(["get"]).unwrap_err();
     }
 
     #[test]
     fn space_in_url() {
-        let cli = parse(&["post", "example.org/foo bar"]).unwrap();
+        let cli = parse(["post", "example.org/foo bar"]).unwrap();
         assert_eq!(cli.method, Some(Method::POST));
         assert_eq!(cli.url.to_string(), "http://example.org/foo%20bar");
         assert!(cli.request_items.items.is_empty());
@@ -1070,74 +1307,82 @@ mod tests {
 
     #[test]
     fn url_with_leading_double_slash_colon() {
-        let cli = parse(&["://example.org"]).unwrap();
+        let cli = parse(["://example.org"]).unwrap();
         assert_eq!(cli.url.to_string(), "http://example.org/");
     }
 
     #[test]
     fn url_with_leading_colon() {
-        let cli = parse(&[":3000"]).unwrap();
+        let cli = parse([":3000"]).unwrap();
         assert_eq!(cli.url.to_string(), "http://localhost:3000/");
 
-        let cli = parse(&[":3000/users"]).unwrap();
+        let cli = parse([":3000/users"]).unwrap();
         assert_eq!(cli.url.to_string(), "http://localhost:3000/users");
 
-        let cli = parse(&[":"]).unwrap();
+        let cli = parse([":"]).unwrap();
         assert_eq!(cli.url.to_string(), "http://localhost/");
 
-        let cli = parse(&[":/users"]).unwrap();
+        let cli = parse([":/users"]).unwrap();
         assert_eq!(cli.url.to_string(), "http://localhost/users");
     }
 
     #[test]
     fn url_with_scheme() {
-        let cli = parse(&["https://example.org"]).unwrap();
+        let cli = parse(["https://example.org"]).unwrap();
         assert_eq!(cli.url.to_string(), "https://example.org/");
     }
 
     #[test]
     fn url_without_scheme() {
-        let cli = parse(&["example.org"]).unwrap();
+        let cli = parse(["example.org"]).unwrap();
         assert_eq!(cli.url.to_string(), "http://example.org/");
     }
 
     #[test]
     fn request_items() {
-        let cli = parse(&["get", "example.org", "foo=bar"]).unwrap();
+        let cli = parse(["get", "example.org", "foo=bar"]).unwrap();
         assert_eq!(cli.method, Some(Method::GET));
         assert_eq!(cli.url.to_string(), "http://example.org/");
         assert_eq!(
             cli.request_items.items,
-            vec![RequestItem::DataField("foo".to_string(), "bar".to_string())]
+            vec![RequestItem::DataField {
+                key: "foo".to_string(),
+                raw_key: "foo".to_string(),
+                value: "bar".to_string()
+            }]
         );
     }
 
     #[test]
     fn request_items_implicit_method() {
-        let cli = parse(&["example.org", "foo=bar"]).unwrap();
+        let cli = parse(["example.org", "foo=bar"]).unwrap();
         assert_eq!(cli.method, None);
         assert_eq!(cli.url.to_string(), "http://example.org/");
         assert_eq!(
             cli.request_items.items,
-            vec![RequestItem::DataField("foo".to_string(), "bar".to_string())]
+            vec![RequestItem::DataField {
+                key: "foo".to_string(),
+                raw_key: "foo".to_string(),
+                value: "bar".to_string()
+            }]
         );
     }
 
     #[test]
     fn request_type_overrides() {
-        let cli = parse(&["--form", "--json", ":"]).unwrap();
+        let cli = parse(["--form", "--json", ":"]).unwrap();
         assert_eq!(cli.request_items.body_type, BodyType::Json);
         assert_eq!(cli.json, true);
         assert_eq!(cli.form, false);
         assert_eq!(cli.multipart, false);
 
-        let cli = parse(&["--json", "--form", ":"]).unwrap();
+        let cli = parse(["--json", "--form", ":"]).unwrap();
         assert_eq!(cli.request_items.body_type, BodyType::Form);
         assert_eq!(cli.json, false);
         assert_eq!(cli.form, true);
         assert_eq!(cli.multipart, false);
 
-        let cli = parse(&[":"]).unwrap();
+        let cli = parse([":"]).unwrap();
         assert_eq!(cli.request_items.body_type, BodyType::Json);
         assert_eq!(cli.json, false);
         assert_eq!(cli.form, false);
@@ -1146,22 +1391,22 @@ mod tests {
 
     #[test]
     fn superfluous_arg() {
-        parse(&["get", "example.org", "foobar"]).unwrap_err();
+        parse(["get", "example.org", "foobar"]).unwrap_err();
     }
 
     #[test]
     fn superfluous_arg_implicit_method() {
-        parse(&["example.org", "foobar"]).unwrap_err();
+        parse(["example.org", "foobar"]).unwrap_err();
     }
 
     #[test]
     fn multiple_methods() {
-        parse(&["get", "post", "example.org"]).unwrap_err();
+        parse(["get", "post", "example.org"]).unwrap_err();
     }
 
     #[test]
     fn proxy_invalid_protocol() {
-        Cli::try_parse_from(&[
+        Cli::try_parse_from([
             "xh",
             "--proxy=invalid:http://127.0.0.1:8000",
             "get",
@@ -1172,13 +1417,13 @@ mod tests {
 
     #[test]
     fn proxy_invalid_proxy_url() {
-        Cli::try_parse_from(&["xh", "--proxy=http:127.0.0.1:8000", "get", "example.org"])
+        Cli::try_parse_from(["xh", "--proxy=http:127.0.0.1:8000", "get", "example.org"])
             .unwrap_err();
     }
 
     #[test]
     fn proxy_http() {
-        let proxy = parse(&["--proxy=http:http://127.0.0.1:8000", "get", "example.org"])
+        let proxy = parse(["--proxy=http:http://127.0.0.1:8000", "get", "example.org"])
             .unwrap()
             .proxy;
 
@@ -1190,7 +1435,7 @@ mod tests {
 
     #[test]
     fn proxy_https() {
-        let proxy = parse(&["--proxy=https:http://127.0.0.1:8000", "get", "example.org"])
+        let proxy = parse(["--proxy=https:http://127.0.0.1:8000", "get", "example.org"])
             .unwrap()
             .proxy;
 
@@ -1202,7 +1447,7 @@ mod tests {
 
     #[test]
     fn proxy_all() {
-        let proxy = parse(&["--proxy=all:http://127.0.0.1:8000", "get", "example.org"])
+        let proxy = parse(["--proxy=all:http://127.0.0.1:8000", "get", "example.org"])
             .unwrap()
             .proxy;
 
@@ -1214,66 +1459,66 @@ mod tests {
 
     #[test]
     fn executable_name() {
-        let args = Cli::try_parse_from(&["xhs", "example.org"]).unwrap();
+        let args = Cli::try_parse_from(["xhs", "example.org"]).unwrap();
         assert_eq!(args.https, true);
     }
 
     #[test]
     fn executable_name_extension() {
-        let args = Cli::try_parse_from(&["xhs.exe", "example.org"]).unwrap();
+        let args = Cli::try_parse_from(["xhs.exe", "example.org"]).unwrap();
         assert_eq!(args.https, true);
     }
 
     #[test]
     fn negated_flags() {
-        let cli = parse(&["--no-offline", ":"]).unwrap();
+        let cli = parse(["--no-offline", ":"]).unwrap();
         assert_eq!(cli.offline, false);
 
         // In HTTPie, the order doesn't matter, so this would be false
-        let cli = parse(&["--no-offline", "--offline", ":"]).unwrap();
+        let cli = parse(["--no-offline", "--offline", ":"]).unwrap();
         assert_eq!(cli.offline, true);
 
         // In HTTPie, this resolves to json, but that seems wrong
-        let cli = parse(&["--no-form", "--multipart", ":"]).unwrap();
+        let cli = parse(["--no-form", "--multipart", ":"]).unwrap();
         assert_eq!(cli.request_items.body_type, BodyType::Multipart);
         assert_eq!(cli.json, false);
         assert_eq!(cli.form, false);
         assert_eq!(cli.multipart, true);
 
-        let cli = parse(&["--multipart", "--no-form", ":"]).unwrap();
+        let cli = parse(["--multipart", "--no-form", ":"]).unwrap();
         assert_eq!(cli.request_items.body_type, BodyType::Multipart);
         assert_eq!(cli.json, false);
         assert_eq!(cli.form, false);
         assert_eq!(cli.multipart, true);
 
-        let cli = parse(&["--form", "--no-form", ":"]).unwrap();
+        let cli = parse(["--form", "--no-form", ":"]).unwrap();
         assert_eq!(cli.request_items.body_type, BodyType::Json);
         assert_eq!(cli.json, false);
         assert_eq!(cli.form, false);
         assert_eq!(cli.multipart, false);
 
-        let cli = parse(&["--form", "--json", "--no-form", ":"]).unwrap();
+        let cli = parse(["--form", "--json", "--no-form", ":"]).unwrap();
         assert_eq!(cli.request_items.body_type, BodyType::Json);
         assert_eq!(cli.json, true);
         assert_eq!(cli.form, false);
         assert_eq!(cli.multipart, false);
 
-        let cli = parse(&["--curl-long", "--no-curl-long", ":"]).unwrap();
+        let cli = parse(["--curl-long", "--no-curl-long", ":"]).unwrap();
         assert_eq!(cli.curl_long, false);
-        let cli = parse(&["--no-curl-long", "--curl-long", ":"]).unwrap();
+        let cli = parse(["--no-curl-long", "--curl-long", ":"]).unwrap();
         assert_eq!(cli.curl_long, true);
 
-        let cli = parse(&["-do=fname", "--continue", "--no-continue", ":"]).unwrap();
+        let cli = parse(["-do=fname", "--continue", "--no-continue", ":"]).unwrap();
         assert_eq!(cli.resume, false);
-        let cli = parse(&["-do=fname", "--no-continue", "--continue", ":"]).unwrap();
+        let cli = parse(["-do=fname", "--no-continue", "--continue", ":"]).unwrap();
         assert_eq!(cli.resume, true);
 
-        let cli = parse(&["-I", "--no-ignore-stdin", ":"]).unwrap();
+        let cli = parse(["-I", "--no-ignore-stdin", ":"]).unwrap();
         assert_eq!(cli.ignore_stdin, false);
-        let cli = parse(&["--no-ignore-stdin", "-I", ":"]).unwrap();
+        let cli = parse(["--no-ignore-stdin", "-I", ":"]).unwrap();
         assert_eq!(cli.ignore_stdin, true);
 
-        let cli = parse(&[
+        let cli = parse([
             "--proxy=http:http://foo",
             "--proxy=http:http://bar",
             "--no-proxy",
@@ -1282,7 +1527,7 @@ mod tests {
         .unwrap();
         assert!(cli.proxy.is_empty());
 
-        let cli = parse(&[
+        let cli = parse([
             "--no-proxy",
             "--proxy=http:http://foo",
             "--proxy=https:http://bar",
@@ -1297,7 +1542,7 @@ mod tests {
             ]
         );
 
-        let cli = parse(&[
+        let cli = parse([
             "--proxy=http:http://foo",
             "--no-proxy",
             "--proxy=https:http://bar",
@@ -1306,13 +1551,13 @@ mod tests {
         .unwrap();
         assert_eq!(cli.proxy, vec![Proxy::Https("http://bar".parse().unwrap())]);
 
-        let cli = parse(&["--bearer=baz", "--no-bearer", ":"]).unwrap();
+        let cli = parse(["--bearer=baz", "--no-bearer", ":"]).unwrap();
         assert_eq!(cli.bearer, None);
 
-        let cli = parse(&["--style=solarized", "--no-style", ":"]).unwrap();
+        let cli = parse(["--style=solarized", "--no-style", ":"]).unwrap();
         assert_eq!(cli.style, None);
 
-        let cli = parse(&[
+        let cli = parse([
             "--auth=foo:bar",
             "--auth-type=bearer",
             "--no-auth-type",
@@ -1325,19 +1570,19 @@ mod tests {
 
     #[test]
     fn negating_check_status() {
-        let cli = parse(&[":"]).unwrap();
+        let cli = parse([":"]).unwrap();
         assert_eq!(cli.check_status, None);
 
-        let cli = parse(&["--check-status", ":"]).unwrap();
+        let cli = parse(["--check-status", ":"]).unwrap();
         assert_eq!(cli.check_status, Some(true));
 
-        let cli = parse(&["--no-check-status", ":"]).unwrap();
+        let cli = parse(["--no-check-status", ":"]).unwrap();
         assert_eq!(cli.check_status, Some(false));
 
-        let cli = parse(&["--check-status", "--no-check-status", ":"]).unwrap();
+        let cli = parse(["--check-status", "--no-check-status", ":"]).unwrap();
         assert_eq!(cli.check_status, Some(false));
 
-        let cli = parse(&["--no-check-status", "--check-status", ":"]).unwrap();
+        let cli = parse(["--no-check-status", "--check-status", ":"]).unwrap();
         assert_eq!(cli.check_status, Some(true));
     }
 
