@@ -116,7 +116,7 @@ pub struct Printer {
 
 impl Printer {
     pub fn new(pretty: Pretty, theme: Option<Theme>, stream: bool, buffer: Buffer) -> Self {
-        let theme = theme.unwrap_or(Theme::auto);
+        let theme = theme.unwrap_or(Theme::Auto);
 
         Printer {
             indent_json: pretty.format(),
@@ -324,6 +324,7 @@ impl Printer {
             header_string.push_str(": ");
             match value.to_str() {
                 Ok(value) => header_string.push_str(value),
+                #[allow(clippy::format_push_string)]
                 Err(_) => header_string.push_str(&format!("{:?}", value)),
             }
             header_string.push('\n');
@@ -784,7 +785,7 @@ mod tests {
         let p = Printer {
             indent_json: false,
             color: false,
-            theme: Theme::auto,
+            theme: Theme::Auto,
             sort_headers: false,
             stream: false,
             buffer: Buffer::new(false, None, false).unwrap(),
